#!/usr/bin/perl
# 
# $Id: fe-rracclk.pl 3494 2010-04-09 09:30:11Z pleiter $
#---------------------------------------------------------------------------------------------------

use strict;
use Getopt::Long;

# Front-End script version
my $VERSION = "1.0";

# Usage
my $USAGE = << 'EOM';
Front-end script version $VERSION

Usage:
 fe-rracclk.pl <RC> get [<node0> <node1> ...]      Readout RRAC clock setting in VPD
 fe-rracclk.pl <RC> set=<clk> [<node0> ...]        Set RRAC clock value in VPD

<RC> is the hostname or IP address of the rootcard.
Node numbers must be specified as values between 0 and 15
If no node numers are given, all 16 nodes are used.
EOM

# Map
my %map = (
  "02" => 200,
  "15" => 210,
  "16" => 220,
  "17" => 230,
  "18" => 240,
  "19" => 250,
  "03" => 300,
);

# Command line options
use vars qw($opt_help);

GetOptions("help");

if (defined $opt_help) {
  print $USAGE;
  exit(0);
}

die "Environment variable QROOT not defined\n" unless defined $ENV{QROOT};

# Get the root card IP address (or corresponding hostname)
# e.g. RCIP=10.98.232.101
my $RCIP = shift @ARGV;

# Get the action that should be executed
my $ACTION = shift @ARGV;

# Get the node numbers
my @NODES = ();
if (scalar @ARGV) {
  @NODES = @ARGV;
}
else {
  @NODES = (0..15);
}

foreach my $i (0..$#NODES) {
  $NODES[$i] = sprintf "%02x", $NODES[$i];
}

# map clock value to clock setting
sub val2key($)
{
  my $val = sprintf "%d", $_[0];
  foreach my $key (keys %map) {
    return $key if ($map{$key} == $val);
  }
  return undef;
}

# map clock setting to clock value
sub key2val($)
{
  my $key = sprintf "%02x", hex($_[0]);
  return $map{$key};
}

# Send command to a Service Processor
# Param $1 = node, Param $2... = SP command bytes
sub feensp
{
  my $cmd = "$ENV{QROOT}/bin/feentest -a $RCIP -t service " . join(" ", @_);

  chomp(my $output=`$cmd 2> /dev/null`);

  # Is status not OK?
  if ($output =~ /RS485_Timeout/) {
    # Timeout, service processor still busy
    print STDERR "ERROR (TIMEOUT)\n";
  }
  elsif ($output =~ /STAT:  fa/) {
    # Service processor is currently busy
    print STDERR "ERROR (BUSY)\n";
  }
  elsif ($output =~ /error/i) {
    # Error from feentest
    print STDERR "ERROR\n";
    print STDERR " $cmd\n";
    print STDERR "$output\n\n";
  }
  elsif ($output !~ /STAT:  00/) {
    # Error from service processor
    print STDERR "ERROR\n";
    print STDERR " $cmd\n";
    print STDERR "$output\n\n";
  }
  else {
    # Command executed successfully
    if ($output =~ /DATA:\s+([^\s]*)/) {
      return "$1";
    }
    else {
      return "OK";
    }
  }

  return undef;
}


sub showconfig()
{
  print "FrontEnd script version $VERSION\n";
  print "RootCard IP: $RCIP\n";
  print "Nodes: " . join(" ", @NODES) . "\n";
}

# VPD-read to read RRAC clock settings from VPD:
sub get
{
  foreach my $n (@NODES) {
    print "Node $n: RRAC clock: ";
    my $key = feensp("$n 31 00 15 01");
    if (defined $key) {
      printf "%d MHz\n", key2val($key);
    }
    else {
      printf "\n";
    }
  }
}

# VPD-write to set RRAC clock value in VPD:
sub set
{
  my ($rracclk) = @_;

  my $key = val2key($rracclk);
  die "Unknown clock setting ($rracclk)\n" unless defined $key;

  foreach my $n (@NODES) {
    print "Node $n: Setting RRAC clock ...\n";
    feensp("$n 32 00 15 01 $key");
  }
}


# Main
for ($ACTION) {
  /^get$/ &&	do {
		  get();
		  last;
		};

  /^set=(\d+)$/ && do {
		  my $rracclk = $1;
		  showconfig();
		  printf "Setting RRAC clk to $rracclk MHz.\n";
		  set($rracclk);
		  last;
		};

  # default
  print $USAGE;
  exit(0);
}
